@@ -70,7 +70,6 @@ def build_raxml(aln_file, out_file, clean_up=True, nthreads=1, tree_builder_args
         if clean_up:
             os.remove("RAxML_bestTree.%s"%(random_string))
             os.remove("RAxML_info.%s"%(random_string))
-            os.remove("RAxML_log.%s"%(random_string))
             os.remove("RAxML_parsimonyTree.%s"%(random_string))
             os.remove("RAxML_result.%s"%(random_string))
 
@@ -118,8 +117,6 @@ def build_fasttree(aln_file, out_file, clean_up=True, nthreads=1, tree_builder_a
     try:
         run_shell_command(cmd, raise_errors = True, extra_env = extra_env)
         T = Phylo.read(out_file, 'newick')
-        if clean_up:
-            os.remove(log_file)
     except:
         print("ERROR: TREE BUILDING FAILED")
         if os.path.isfile(log_file):
@@ -142,8 +139,11 @@ def build_iqtree(aln_file, out_file, substitution_model="GTR", clean_up=True, nt
     # IQ-tree messes with taxon names. Hence remove offending characters, reinstaniate later
     tmp_aln_file = aln_file.replace(".fasta", "-delim.fasta")
     log_file = tmp_aln_file.replace(".fasta", ".iqtree.log")
+    num_seqs = 0
     with open(tmp_aln_file, 'w', encoding='utf-8') as ofile:
         for line in tmp_seqs:
+            if line.startswith(">"):
+                num_seqs += 1
             ofile.write(line.replace('/', '_X_X_').replace('|','_Y_Y_').replace("(","_X_Y_").replace(")","_Y_X_"))
 
     # For compat with older versions of iqtree, we avoid the newish -fast
@@ -163,6 +163,19 @@ def build_iqtree(aln_file, out_file, substitution_model="GTR", clean_up=True, nt
         "-me",    "0.05"
     ]
 
+    # Use IQ-TREE's auto-scaling of threads when the user has requested more
+    # threads than there are sequences. This approach avoids an error from
+    # IQ-TREE when num_seq < nthreads (as when users request `-nthreads auto` on
+    # a machine with many cores and fewer input sequences) and also avoids
+    # requesting as many threads as there are sequences when there may be fewer
+    # available threads on the current machine.
+    if num_seqs < nthreads:
+        nthreads = "AUTO"
+        print(
+            "WARNING: more threads requested than there are sequences; falling back to IQ-TREE's `-nt AUTO` mode.",
+            file=sys.stderr
+        )
+
     if substitution_model.lower() != "none":
         call = ["iqtree", *fast_opts, "-nt", str(nthreads), "-s", shquote(tmp_aln_file),
                 "-m", substitution_model, tree_builder_args, ">", log_file]
@@ -189,11 +202,10 @@ def build_iqtree(aln_file, out_file, substitution_model="GTR", clean_up=True, nt
             if substitution_model.lower() == 'none':
                 shutil.copyfile(log_file, out_file.replace(out_file.split('/')[-1],"iqtree.log"))
 
-            for f in [log_file, tmp_aln_file]:
-                if os.path.isfile(f):
-                    os.remove(f)
+            if os.path.isfile(tmp_aln_file):
+                os.remove(tmp_aln_file)
 
-            for ext in [".bionj",".ckp.gz",".iqtree",".log",".mldist",".model.gz",".treefile",".uniqueseq.phy",".model"]:
+            for ext in [".bionj",".ckp.gz",".iqtree",".mldist",".model.gz",".treefile",".uniqueseq.phy",".model"]:
                 if os.path.isfile(tmp_aln_file + ext):
                     os.remove(tmp_aln_file + ext)
     except:
@@ -214,7 +226,7 @@ def write_out_informative_fasta(compress_seq, alignment, stripFile=None):
     positions = compress_seq['positions']
 
     #If want to exclude sites from initial treebuild, read in here
-    strip_pos = load_mask_sites(stripFile)
+    strip_pos = load_mask_sites(stripFile) if stripFile else []
 
     #Get sequence names
     seqNames = list(sequences.keys())
